%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% sample_cop_theta_prior:
%
% Generate samples from the prior distribution for copula parameters. In
% this case, a truncated double exponential or normal.
%
% Input:
%
% - M: number of samples
% - cop_hyper: hyperparameter
% - cutoff: symmetric boundary of the sample space
% - use_laplace: double exponential version of the prior
%
% Output:
% 
% - theta: the samples
%
% Created by: Ricardo Silva, London, 14/03/2012
% University College London
% Current version: 14/03/2012

function theta = sample_cop_theta_prior(M, cop_hyper, cutoff, use_laplace)

if nargin == 4 && use_laplace
    
  signs = 2 * (rand(M, 1) > 0.5) - 1; % Signs
  theta = exprnd(cop_hyper, M, 1);
  idx = find(theta > cutoff);

  while ~isempty(idx)
    theta(idx) = exprnd(cop_hyper, length(idx), 1);
    idx = find(theta > cutoff);
  end

  theta = theta .* signs;
  
else
    
  theta = randn(M, 1) * cop_hyper;
  idx = find(abs(theta) > cutoff);

  while ~isempty(idx)
    theta(idx) = randn(length(idx), 1) * cop_hyper;
    idx = find(abs(theta) > cutoff);
  end
  
end
