%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% binary_sufficient:
% 
% Compute sufficient statistics for all pairs in the multivariate binary
% case.
%
% Input:
%
% - Y: the data, encoded as {-1, 1}
%
% Output:
% 
% - Y_counts: indicates counts for each combination of values
%
% Created by: Ricardo Silva, London, 03/02/2012
% University College London
% Current version: 03/02/2012

function Y_counts = binary_sufficient(Y)

num_y = size(Y, 2);

Y_counts = cell(2, 2);

for i = 1:2
  for j = 1:2
    Y_counts{i, j} = zeros(num_y);
  end
end

for y1 = 1:num_y
  for y2 = (y1 + 1):num_y
    for i = 1:2
      i_v = (i - 1) * 2 - 1;
      for j = 1:2
        j_v = (j - 1) * 2 - 1;  
        Y_counts{i, j}(y1, y2) = sum((Y(:, y1) == i_v) .* (Y(:, y2) == j_v));
        Y_counts{j, i}(y2, y1) = Y_counts{i, j}(y1, y2);
      end
    end
  end
end
