%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% build_x_weight_library:
%
% Generate latent variable assignments and all possible weights for a range
% of possible correlations.
%
% Input:
%
% - num_x: number of latent points
% - rho_domain: possible values of rho
%
% Output:
% 
% - X: latent points
% - w: corresponding weights, organized as a matrix
%
% Created by: Ricardo Silva, London, 08/03/2012
% University College London
% Current version: 08/03/2012

function [X ws] = build_x_weight_library(N, rho_domain)

l = -3; t = 3;
interval = (t - l) / N;

x_quant_range = l:interval:t;
x_range = x_quant_range;

X  = zeros(N^2, 2);
ws = zeros(N^2, length(rho_domain));

x_count = 1;
for x1 = 1:N
  for x2 = 1:N
    xu1 = x_range(x1 + 1); xl1 = x_range(x1); xv1 = (xl1 + xu1) / 2;
    xu2 = x_range(x2 + 1); xl2 = x_range(x2); xv2 = (xl2 + xu2) / 2;
    X(x_count, 1) = xv1; X(x_count, 2) = xv2; 
    x_count = x_count + 1;
  end
end

for rho_idx = 1:length(rho_domain)
  rho = rho_domain(rho_idx);
  fprintf('rho = %f\n', rho);
  S = [1 rho; rho 1];
  x_count = 1;
  for x1 = 1:N
    for x2 = 1:N
      xu1 = x_range(x1 + 1); xl1 = x_range(x1);
      xu2 = x_range(x2 + 1); xl2 = x_range(x2);
      cdfs = mvncdf([xu1 xu2; xl1 xu2; xu1 xl2; xl1 xl2], [0 0], S);
      ws(x_count, rho_idx) = cdfs(1) - cdfs(2) - cdfs(3) + cdfs(4);
      x_count = x_count + 1;
    end
  end
end
