%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% cdf_probit:
%
% Transform data passing it through marginals CDFs as given by a probit
% model. This is conditioned on particular values of the latent variables.
%
% Input:
%
% - L: loading coefficients
% - X_parents: the data
%   
% Output:
% 
% - U: transformed marginals
%
% Created by: Ricardo Silva, London, 21/02/2012
% University College London
% Current version: 21/02/2012

function U = cdf_probit(L, X_parents, X_values)

num_x_v = size(X_values, 1);
num_y = size(L, 1);

U = zeros(num_x_v, num_y);

for x = 1:num_x_v
  U(x, :) = normcdf(0, L(:, 1) .* X_values(x, X_parents)' + L(:, 2), ...
                       ones(num_y, 1));
end

