%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% sample_archimedean:
%
% Given latent variables, sample for a latent variable copula model using 
% the Laplace transform technique. Currently only for Frank models with 
% positive copula parameters.
%
% Input:
%
% - L, expo_y, cop_theta: parameters of the model
% - Y_parents: information about directed structure
% - G: bidirected structure
% - X_data: sampled values for the latent variables
%
% Output:
% 
% - Y: binary data in 0/1 format
%
% Created by: Ricardo Silva, London, 18/03/2012
% University College London
% Current version: 18/03/2012

function Y = sample_archimedean(L, expo_y, cop_theta, Y_parents, G, X_data)

%cop_theta = 5 * ones(size(cop_theta)); %TODO REMOVE THIS

num_y = size(L, 1);
N = size(X_data, 1);

Y = zeros(N, num_y);

edges = graph_edge_list(G); num_edges = size(edges, 1);
V = zeros(N, num_edges);
k = 1:15000000; % Truncate at 500000 to make things "simple"
for i = 1:num_edges  
  theta = cop_theta(i);
  p = (1 - exp(-theta)).^k ./ (k * theta);
  V(:, i) = randsample(length(k), N, true, p);
end

U_sample = zeros(N, num_y);
for y = 1:num_y
  fprintf('Generating data for variable %d\n', y);
  num_edges_y = sum(G(:, y));
  if num_edges_y == 0
    U_sample(:, y) = rand(N, 1);
    continue
  end
  edges_y = zeros(num_edges_y, 1); count_e = 1;
  neighbors = zeros(num_edges_y, 1);
  for i = 1:num_edges
    if edges(i, 1) == y || edges(i, 2) == y
      edges_y(count_e) = i;
      if edges(i, 1) ~= y
        neighbors(count_e) = edges(i, 1);
      else
        neighbors(count_e) = edges(i, 2);
      end
      count_e = count_e + 1;
    end        
  end
  U_sample(:, y) = sample_u(cop_theta(edges_y), expo_y(y, neighbors), ...
                            V(:, edges_y));  
end
for y = 1:num_y
  U = normcdf(0, L(y, 1) * X_data(:, Y_parents(y)) + L(y, 2), 1);
  Y(U_sample(:, y) > U, y) = 1;
end

    function y = generator(x, theta)
      y = -log((exp(-theta * x) - 1) / (exp(-theta) - 1));
    end

    function output = sample_u(sub_theta, sub_exp_theta, sub_V)      
      N_V = size(sub_V, 1);              
      output = zeros(N_V, 1);
      for n = 1:N_V
        x = rand();
        output(n) = binsearch_invert(x);
        %output(n) = -(log(exp(log(x) / sub_V(n, 1)) * (exp(-sub_theta(1)) - 1) + 1)) / sub_theta(1);
      end
      
        function v = binsearch_invert(x)
          a = 0; b = 1;
          while true
            val = (a + b) / 2;
            f = 1;
            for e = 1:length(sub_theta)
              f = f * exp(-generator(val^sub_exp_theta(e), sub_theta(e)) * ...
                          sub_V(n, e));
            end
            if abs(f - x) < 1.e-4, break; end
            if f < x
              a = val;
            else
              b = val;
            end          
          end
          v = val;
        end
    end

end
